<?php
/**
 * Tools for DMG Tolkien integration
 *
 * @package DMG\Tolkien_Plugin
 */

/**
 * Plugin Name: DMG Tolkien
 * Plugin URI:  https://github.com/ANMedia/dmg-tolkien-wp-plugin
 * Description: Utilites to support DMG Tolkien
 * Version:     1.0.6
 * Author:      markparolisi
 */

namespace DMG;

/**
 * Utilities for DMG Tolkien integration
 */
class Tolkien_Plugin {


	/**
	 * Domain of the tolkien API service
	 *
	 * @var string
	 */
	public static $tolkien_api_domain = 'https://tolkien.anm.co.uk';

	/**
	 * List of post types to send to tolkien
	 *
	 * @var array
	 */
	public static $post_types = array( 'post' );

	/**
	 * Bootstrap callbacks
	 *
	 * @return void
	 */
	public static function init() {

		// @TODO use other actions? Make this action filterable?
		add_action( 'wp_after_insert_post', array( __CLASS__, 'post_updated_callback' ), 10, 1 );
		
		// Register a new cron interval
		add_filter( 'cron_schedules', function($schedules){
			$schedules['fifteen_minutes'] = [
				'interval' => 15 * MINUTE_IN_SECONDS,
				'display' => 'Once every fifteen minutes',
			];
			return $schedules;
		} );

		// Register the cron
		add_action( 'admin_init', function(){
			if (! wp_next_scheduled ( 'dmg_tolkien_recent_error_cron' )) {
				wp_schedule_event( time(), 'fifteen_minutes', 'dmg_tolkien_recent_error_cron' );
			}
		});

		// Register the callback for the cron
		add_action( 'dmg_tolkien_recent_error_cron', [ __CLASS__, 'collect_recent_errors' ], 10, 2 );
		if ( defined( 'WPCOM_IS_VIP_ENV' ) && true === WPCOM_IS_VIP_ENV ) {
			add_filter( 'wpcom_vip_passthrough_cron_to_jobs', [ __CLASS__, 'passthrough_cron_to_jobs' ] );
		}
		
		add_action( 'rest_api_init', function () {
			register_rest_route( 'wp/v2', '/tolkien/errors', array(
				'methods' => 'GET',
				'callback' => [__CLASS__, 'api_error_log_callback'],
			) );
		} );

		self::$post_types         = apply_filters( 'dmg_tolkien_post_types', self::$post_types );
		self::$tolkien_api_domain = apply_filters( 'dmg_tolkien_api_domain', self::$tolkien_api_domain );
	}

	/**
	 * VIP Cron override
	 *
	 * @param array $hooks The current hooks
	 * @return array Updated hooks
	 */
	public static function passthrough_cron_to_jobs( $hooks ) {
		$hooks[] = 'dmg_tolkien_recent_error_cron';
		return $hooks;
	}

	/**
	 * Generate the Schema.org NewsArticle data to send to tolkien
	 *
	 * @param int $post_id The Post ID
	 * @return array The schema.org JSON+LD object
	 */
	public static function generate_schema( $post_id ) {
		$schema = array();

		if ( function_exists( 'YoastSEO' ) ) {

			$yoast       = \YoastSEO();
			$service     = $yoast->classes->get( \Yoast\WP\SEO\Actions\Indexables\Indexable_Head_Action::class );
			$post_data   = $service->for_post( $post_id );
			$full_schema = $post_data->json['schema'];

			// filter to only NewsArticle
			foreach ( $full_schema['@graph'] as $schema_partial ) {
				if ( in_array($schema_partial['@type'], ['NewsArticle', 'WebPage'], true ) ) {
					$schema = $schema_partial;

					$schema['identifier'] = $post_id;

					$schema['creativeWorkStatus'] = get_post_status( $post_id );

					// hard code the full featured image to both
					$featured_image_url     = get_the_post_thumbnail_url( $post_id, 'full' );
					$featured_image_url     = ( ! empty( $featured_image_url ) ) ? $featured_image_url : '';
					$schema['thumbnailUrl'] = $featured_image_url;
					$schema['image']        = $featured_image_url;

					// make sure certain props are populated
					if ( empty( $schema['headline'] ) ) {
						$schema['headline'] = get_the_title( $post_id );
					}

					if ( empty( $schema['articleBody'] ) ) {
						$schema['articleBody'] = apply_filters( 'the_content', strip_shortcodes( get_the_content( null, false, $post_id ) ) );
					}

					if ( empty( $schema['abstract'] ) ) {
						$schema['abstract'] = wp_strip_all_tags( get_the_excerpt( $post_id ) );
					}

					if ( empty( $schema['alternativeHeadline'] ) ) {
						$schema['alternativeHeadline'] = wp_strip_all_tags( get_post_meta( $post_id, '_yoast_wpseo_opengraph-title', true ) );
					}

					// Generate a created date as the date of the first revision
					if ( empty( $schema['dateCreated'] ) ) {
						$args      = array(
							'order' => 'ASC',
						);
						$revisions = wp_get_post_revisions( $post_id, $args );
						if ( ! empty( $revisions ) ) {
							$first                 = $revisions[ min( array_keys( $revisions ) ) ];
							$schema['dateCreated'] = get_the_date( 'c', $first->ID );
						}
					}

					// Simplify the publisher attribute from the Organization reference to just the site title, e.g. "Metro"
					$schema['publisher'] = get_bloginfo( 'name' );

					// Re-order the articleSection categories as [channel,subChannel]
					$cat = get_the_category( $post_id );
					if ( ! empty( $cat ) && $cat[0]->category_parent ) {
						$cat_parent = get_category( $cat[0]->category_parent );
						$extra_cats = '';
						while ( 0 !== $cat_parent->category_parent ) {
							$extra_cats = $cat_parent->slug . '/' . $extra_cats;
							$cat_parent = get_category( $cat_parent->category_parent );
						}
						$parent_category          = $cat_parent->slug;
						$schema['articleSection'] = array( $parent_category, $cat[0]->slug );
					} else {
						$schema['articleSection'] = array( $cat[0]->slug );
					}

					// Check for coauthors
					if ( function_exists( 'get_coauthors' ) ) {
						$coauthors = \get_coauthors( $post_id );
						if ( ! empty( $coauthors ) ) {
							$author_names     = array_map(
								function( $coauthor ) {
									return $coauthor->display_name;
								},
								$coauthors
							);
							$schema['author'] = array();
							foreach ( $author_names as $author_name ) {
								$schema['author'][] = array(
									'@type' => 'Person',
									'name'  => $author_name,
								);
							}
						}
					}

					if ( is_array( $schema['keywords'] ) ) {
						$schema['keywords'] = implode( ',', $schema['keywords'] );
					}
				}
			}
		}

		$schema = (array) apply_filters( 'dmg_tolkien_schema', $schema, $post_id );
		// sort the array by key name
		ksort( $schema );
		return $schema;
	}

	/**
	 * Post the update to the tolkien ingestion service
	 *
	 * @param int $post_id The Post ID
	 * @return void
	 */
	protected static function send_update( $post_id ) {

		$schema = self::generate_schema( $post_id );

		// Exit if no schema can be generated.
		if ( empty( $schema ) ) {
			// phpcs:ignore
			trigger_error( 'failed to generate Tolkein Schema', E_USER_NOTICE );
			return;
		}

		$url = self::$tolkien_api_domain . '/ingest/article';

		// @link https://developer.wordpress.org/reference/classes/WP_Http/request/
		$default_args = array(
			'method'  => 'POST',
			'headers' => array(
				'api-token'    => '5mTXxm5CTFzPuAErDxwgnyqMJmTTAaTU',
				'Content-Type' => 'application/json; charset=utf-8',
			),
			'body'    => wp_json_encode( $schema ),
		);

		$args = apply_filters( 'dmg_tolkien_request_args', $default_args );

		$response = wp_remote_post( $url, $args );

		// set last_updated post_meta for debugging
		update_post_meta( $post_id, 'dmg_tolkien_last_updated', time() );
		$logged_response = ( is_wp_error( $response ) ) ? $response->errors : $response;
		update_post_meta( $post_id, 'dmg_tolkien_last_response', $logged_response );

		return $response;
	}


	/**
	 * Handle when a post is updated
	 *
	 * @param int $post_id The Post ID
	 * @return void
	 */
	public static function post_updated_callback( $post_id ) {

		$send_update = true;

		if ( defined( '\VIP_GO_APP_ENVIRONMENT' ) && 'production' !== \VIP_GO_APP_ENVIRONMENT ) {
			$send_update = false;
		}

		$send_update = apply_filters( 'dmg_tolkien_send_update', $send_update, $post_id );

		if ( !$send_update ) {
			return;
		}

		// Return if not a support post type
		if ( ! in_array( get_post_type( $post_id ), self::$post_types, true ) ) {
			return;
		}

		self::send_update( $post_id );
	}

	/**
	 * Build a list of recent Tolkien errors
	 *
	 * @return array The list of errors
	 */
	public static function collect_recent_errors(){

		$errors = [];

		$query_args = [
			'post_type' => self::$post_types,
			'post_status' => 'any',
			'fields' => 'ids',
			'date_query' => [
				'relation'   => 'OR',
				[
					'column'  => 'post_date',
					'after'   => '-15 min'
				],
				[
					'column'  => 'post_modified',
					'after'   => '-15 min'
				]
			],
		];

		$recent_posts = new \WP_Query( $query_args );

		foreach( $recent_posts->posts as $recent_post ) {
			$tolkein_response = get_post_meta( $recent_post, 'dmg_tolkien_last_response', true );
			if( $tolkein_response ){
				// ensure the last response was an error
				if( !empty( $tolkein_response['response'] ) && $tolkein_response['response']['code'] !== 200 ) {
					$tolkein_response['post_ID'] = $recent_post->ID;
					$tolkein_response['timestamp'] = get_post_meta( $recent_post->ID, 'dmg_tolkien_last_updated', true );
					$errors[] = $tolkein_response;
				}
			}
		}

		update_option( 'dmg_tolkien_recent_errors', $errors );

		return $errors;
	}

	/**
	 * Render the recent errors via an API endpoint for Cockpit to monitor
	 *
	 * @param [type] $data
	 * @return array The list of errors
	 */
	public static function api_error_log_callback($data){

		$errors = (array) get_option( 'dmg_tolkien_recent_errors', [] );
 
  		return $errors;
	}
}

Tolkien_Plugin::init();
